// String support -*- C++ -*-

// Copyright (C) 2001, 2002 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING.  If not, write to the Free
// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.

// As a special exception, you may use this file as part of a free software
// library without restriction.  Specifically, if other files instantiate
// templates or use macros or inline functions from this file, or you compile
// this file and link it with other files to produce an executable, this
// file does not by itself cause the resulting executable to be covered by
// the GNU General Public License.  This exception does not however
// invalidate any other reasons why the executable file might be covered by
// the GNU General Public License.

//
// ISO C++ 14882: 21 Strings library
//

/** @file stringfwd.h
 *  This is an internal header file, included by other library headers.
 *  You should not attempt to use it directly.
 */

#ifndef _CPP_BITS_STRINGFWD_H
#define _CPP_BITS_STRINGFWD_H	1

#pragma GCC system_header

#include <bits/c++config.h>

namespace std
{
  template<class _CharT>
    struct char_traits;
  
  template<> struct char_traits<char>;
#ifdef _GLIBCPP_USE_WCHAR_T
  template<> struct char_traits<wchar_t>;
#endif

  template<typename _Alloc> 
    class allocator;

  template<typename _CharT, typename _Traits = char_traits<_CharT>, 
           typename _Alloc = allocator<_CharT> >
    class basic_string;

  typedef basic_string<char>    string;
  typedef basic_string<wchar_t> wstring;
} // namespace std

#endif	// _CPP_BITS_STRINGFWD_H
